#import <Foundation/NSObject.h>
#import "readOnlyTextStream.h"

/*
The spell-checking functionality can only be performed on text streams which respond to the NXReadOnlyTextStream protocol and to the NXSelectText protocol for dealing with a selection in the text stream on screen (since spell-checking is a necessarily interactive mechanism).
*/
 
@protocol NXSelectText
- (void)selectCharactersFrom:(int)start to:(int)end;
- (int)selectionCharacterCount;
- (int)readCharactersFromSelection:(char *)buffer count:(int)count;
- (void)makeSelectionVisible;
@end

/*
Text streams can optionally respond to this method if they want to support not only spell-checking, but spell correction.  This message is sent down the responder chain.  The receiver should ask the sender for its selectedCell's stringValue (the correct spelling of the word) and replace its selection if appropriate.
*/
 
@protocol NXChangeSpelling
- changeSpelling:sender;
@end

/*
If a document responds to this protocol, then the Ignore button will work in the spell checking panel.  This method must return a unique integer specific to the document.  This identifier can be used with the *ForSpellDocument: methods.  A tag of ZERO is illegal.  If you don't want your document to be able to have document-specific ignored words, then don't implement this method.
*/

@protocol NXIgnoreMisspelledWords
- (int)spellDocumentTag;
@end

/*
The NXSpellChecker object is used by a client (e.g. a document in an application) to spell-check a given NXReadOnlyTextStream (a protocol which makes the text in a document available to other objects via a series of methods).  There is only one NXSpellChecker instance per application (since spell-checking is interactive and you only have one mouse and one keyboard).

The textStream being spell-checked need only be valid for the duration of the call to checkSpelling:of:.  At the start of checkSpelling:of:, the currentCharacterOffset should be equal to the start of the current selection in the text (or the insertion point or the start of the text if there is no selection or insertion point).

The usual usage of this is to implement a checkSpelling: method in an object that responds to the NXReadOnlyTextStream and NXSelectText protocols, then, upon receiving checkSpelling:, the NXReadOnlyTextStream object calls [[NXSpellChecker sharedInstance] checkSpelling:mode of:self].
*/

typedef enum {
    NX_CheckSpelling,
    NX_CheckSpellingToEnd,
    NX_CheckSpellingFromStart,
    NX_CheckSpellingInSelection,
    NX_CountWords,
    NX_CountWordsToEnd,
    NX_CountWordsInSelection
} NXSpellCheckMode;

@interface NSSpellChecker : NSObject
{
@private		/* these are private and are */
    id serverHashTable;	/* subject to change in any future */
    id realTextStream;	/* release, so don't access them! */
    id spellController;
    id guessesBrowser;
    id wordField;
    id languagePopUp;
    id guessesList;
    id panel;
    id checkers;
    id userDictionaries;
    id correctButton;
    id guessButton;
    id ignoreButton;
    id accessoryView;
    int startOffset;
    int selectionOffset;
    int selectionSize;
    id dictionaryBrowser;
    char *selectionBuffer;
    void *_spellServers;
    char *lastGuess;
    BOOL autoShowGuessPanel;
    BOOL needDelayedGuess;
    BOOL haveUsed;
    BOOL reserved3;
    id deleteButton;
    id openButton;
    id learnButton;
    id forgetButton;
    void *reserved;
}

/* Only one per application. */

+ sharedInstance;
+ sharedInstance:(BOOL)create;

/* Initiates a spell-check of the text in textStream.  Returns whether a misspelling was found. */

- (BOOL)checkSpelling:(NXSpellCheckMode)mode of:(id <NXReadOnlyTextStream, NXSelectText>)textStream;
- (BOOL)checkSpelling:(NXSpellCheckMode)mode
		   of:(id <NXReadOnlyTextStream, NXSelectText>)textStream
	    wordCount:(int *)wordCount;
 
/* Sets and gets attributes of the spell-correction panel. */

- spellingPanel;
- accessoryView;
- setAccessoryView:aView;

/* Ignored words.  The caller owns the return value from get and should free it.  When a document closes, it should notify the NXSpellChecker via closeSpellDocumentWithTag: so that its ignored word list gets cleaned up. */

- (char **)ignoredWordsForSpellDocument:(int)tag;
- setIgnoredWords:(const char *const *)words forSpellDocument:(int)tag;
- closeSpellDocument:(int)tag;

/* Allows programmatic setting of the language to spell-check in (normally chosen by a pop-up-list in the spelling panel and defaulted to the user's preferred language, so call this with care). */

- (NSString *)language;
- setLanguage:(NSString *)language;

/* Allows programmatic setting of the misspelled word field. */

- setWordFieldValue:(NSString *)aString;

@end
